from typing import Any

fft: Any
ifft: Any
rfft: Any
irfft: Any
hfft: Any
ihfft: Any
rfftn: Any
irfftn: Any
rfft2: Any
irfft2: Any
fft2: Any
ifft2: Any
fftn: Any
ifftn: Any
fftshift: Any
ifftshift: Any
fftfreq: Any
rfftfreq: Any
