from typing import Any

matrix_power: Any
solve: Any
tensorsolve: Any
tensorinv: Any
inv: Any
cholesky: Any
eigvals: Any
eigvalsh: Any
pinv: Any
slogdet: Any
det: Any
svd: Any
eig: Any
eigh: Any
lstsq: Any
norm: Any
qr: Any
cond: Any
matrix_rank: Any
LinAlgError: Any
multi_dot: Any
